var fs = require('fs'),
	path = require('path'),
	componentBuilder = require('./componentBuilder.js');

module.exports = {

	init: function(conf) {
		this.conf = conf;
		this.componentBuilder = componentBuilder;
		this.componentBuilder.init(conf);
	},

	createTasks: function(gruntHelper, taskName, taskList){
		var themes = this.getThemesFromFiles(this.conf.sources.themes),
			files,themeName,component,tmpPath,jsCode,themeFilePath;

		for (var i = 0; i < themes.length; i++) {
			themeName = themes[i].name;
			themeFilePath = path.join(this.conf.output.css.themes, themeName + '.css')
			jsCode = '';

			// variables + theme vars + theme-base
			files = [
				path.join(this.conf.sources.components, 'variables.css'),
				path.join(this.conf.sources.themes, themeName, 'css', 'variables.css'),
				path.join(this.conf.output.css.base, 'theme-base.css'),

			];

			// theme dependencies
			if(themes[i].dependencies){
				for (var ii = 0; ii < themes[i].dependencies.length; ii++) {
					component = this.componentBuilder.getComponentById(themes[i].dependencies[ii]);
					files = files.concat(this.componentBuilder.getComponentCssDependencyFiles(component, true));

					if(component.dependencies.js){
						console.log('theme '+themeName+' need '+component.id);
						jsCode+= 'sifeComponents[\''+component.id+'\'] = require(\''+path.resolve(component.path)+'/js/'+component.name+'.js\');\n';
					}
				}
			}

			// and finally the theme skin
			files.push(path.join(this.conf.sources.themes, themeName, 'css', 'skin.css'));

			gruntHelper.concat.add(taskName, themeFilePath, files);

			// and extract rules vars
			var extractConfig = {};
			extractConfig[themeName] = {src:[themeFilePath]};
			gruntHelper.setGruntConfig('extractrulesvars', extractConfig);
			taskList.push('extractrulesvars:'+themeName);

			if(jsCode !== ''){
				jsCode = 'var sifeComponents = {};\n' + jsCode + '\n$("[data-sife-js]").each(function(){function e(){n++,n===t.externalSources.length&&t($(this))}var s=$(this).attr("data-sife-js");if(sifeComponents[s]){var t=sifeComponents[s],n=0;if(t.externalSources)for(var r=0;r<t.externalSources.length;r++)t.externalSources[r].indexOf(".css")>-1?loadCSS(t.externalSources[r],null,e.bind(this)):$.getScript(t.externalSources[r],e.bind(this));else t($(this))}});';
				tmpPath = '/tmp/sife_themjs_'+themeName+'.js';
				fs.writeFileSync(tmpPath, jsCode);

				gruntHelper.copy.add(taskName, this.conf.output.js+'/themes/'+themeName+'.js', [tmpPath]);
				taskList.push('copy:'+taskName);

				gruntHelper.browserify.add(taskName, this.conf.output.js+'/themes/'+themeName+'-bundle.js', this.conf.output.js+'/themes/'+themeName+'.js');
				taskList.push('browserify:'+taskName);
			}
		}
	},

	getThemesFromFiles: function(themesPath){
		var dirs = fs.readdirSync(themesPath),
			themes = [],
			theme,themeConfig;
		for (var i = 0; i < dirs.length; i++) {
			themeConfig = this.getThemeConfig(dirs[i]);
			theme = {
				name: dirs[i],
				dependencies: themeConfig.dependencies
			};
			themes.push(theme);
		}
		return themes;
	},

	getThemeConfig: function(themeName){
		var config = {};
		try{
			config = JSON.parse(fs.readFileSync(path.join(this.conf.sources.themes, themeName, 'config.json')));
		}catch(err){
			config = {};
		}
		return config;
	}
};